function K = kernelFunction(X, Y, type, tau)
% This function is used for calculating the kernel matrix.
%
% Input
%       X       --  the training data set stored by cell, in which the c-th
%                   one stands for the data from class c, with size p x Nc;
%       Y       --  the desired reduced dimension;
%       type    --  the kernel function:
%                       'linear'        -- linear kernel;
%                       'GaussianI'     -- Gaussian kernel with modification;
%                       'GaussianII'    -- Gaussian kernel;
%                       'polynomial'    -- polynomial kernel;
%                       'Sigmoid'       -- Sigmoid kernel;
%       tau     --  the parameter for each kernel function;
%
% Output
%       K       -- the kernel matrix
%
% Author: Shu (Aimery) Kong
% Date:   Jan. 19, 2013

if nargin == 2
    type = 'GaussianI';
    tau = 0.1;
elseif nargin < 4
    tau = 0.1;
end

I = size(X, 2);
J = size(Y, 2);
K = zeros( I, J );

if strcmp(type, 'GaussianI')
    tmp = abs(X'*Y);
    K = exp(-1*tmp/tau);
    
%     for j = 1:J
%         tmp = X - repmat( Y(:,j), 1, size(X,2));
%         tmp = tmp.^2;
%         tmp = (sum(tmp,1));
%         tmp = -1.*tmp';
%         K(:,j) = exp(tmp/tau);
%     end
elseif strcmp(type, 'GaussianII')
    for j = 1:J
        tmp = X - repmat( Y(:,j), 1, size(X,2));
        tmp = tmp.^2;
        tmp = (sum(tmp,1)); %sqrt
        tmp = -1.*tmp';
        K(:,j) = exp(tmp/tau);
    end
elseif strcmp(type, 'polynomial')
    K = (ones(size(X,2),size(Y,2)) + X'*Y);
    K = K.^tau;

%     for i = 1:I
%         for j = 1:J
%             K(i,j) = (1 + norm(X(:,i)-Y(:,j)) )^tau;
%         end
%     end
elseif strcmp(type, 'Sigmoid')
    K = tanh(1*X'*Y + tau);
  
%   for i = 1:I
%       for j = 1:J
%           K(i,j) = tanh( norm(X(:,i)-Y(:,j))^2 + tau);
%       end
%   end
elseif strcmp(type, 'linear')
    K = X'*Y;
%     for i = 1:I
%         for j = 1:J
%             K(i,j) = X(:,i)'*Y(:,j);
%         end
%     end
end