% A demo for the paper below:
%
% Shu Kong, Donghui Wang. "Learning Exemplar-Represented Manifolds in
% Latent Space for Classification", ECML/PKDD, 2013.
%
% SLEP_package (http://www.public.asu.edu/~jye02/Software/SLEP/) is
% included here.
%
%
% Author: Shu Kong (Aimery) 
% aimerykong@gmail.com
% Feb. 27, 2014

close all; 
clear; 
clc;

addpath(genpath('.\SLEP_package_4.1\'));

%% parameters
load .\YaleB_32x32_cell.mat; %circle.mat; %sine.mat;%parabola.mat;%
% COIL20_cell.mat  USPS_cell.mat  USPS_subset_cell.mat
% YaleB_32x32_cell.mat

C = round(length(data)/5);
dataNew = cell(1,C);
for c = 1:C
    dataNew{c} = data{c}(:, 1:end);
end
data = dataNew;
clear dataNew;

type = 'linear'; % 'linear' 'Sigmoid' 'polynomial' 'GaussianI' 'GaussianII'
tau = 4; % kernel parameter [7 3 1 0.5 0.1]
lambda = 0.51;
MAXLOOP = 5;
dimP = 3;

% Starting point
opts.init=2;        % starting from a zero point
% Termination
opts.tFlag=5;       % run .maxIter iterations
opts.maxIter=100;   % maximum number of iterations
% Normalization
opts.nFlag=0;       % without normalization
% Regularization
opts.rFlag=1;       % the input parameter 'rho' is a ratio in (0, 1)
% Group Property
opts.q=2;           % set the value for q

opts.mFlag=1; % smooth reformulation
opts.lFlag=1; % adaptive line search

%% initialization of P, such that X*P is the projection
[P, X] = initialize_P(data, dimP, 'LDA', type, tau ); % 'random'
labelList = [];
C = length(data);
for c = 1:C
    labelList = [labelList c*ones(1,size(data{c},2))];
end

%% perform optimization
iter = 1;
A = cell(1,c);
K = X'*X;
residualList = zeros(1, MAXLOOP);
fprintf('begin optimization (MAXLOOP=%d)...\n', MAXLOOP);
while iter <= MAXLOOP
    % update coefficient matrix Ac
    for c = 1:C 
        A{c} = updateAc_alternative(P, X, labelList, c, lambda, opts, type, tau);
    end
    
    % update projection W
    [P, Aall, Ahat, Abar] = updateP(P, X, A, labelList, dimP, type, tau);
    
    % calculate the residuals
    residualList(iter) = norm(P'*K*(eye(size(K,1)) - Aall), 'fro') + norm(P'*K*(eye(size(K,1)) - Ahat), 'fro') + norm(P'*K*Abar, 'fro') + lambda * sum(sqrt(sum(Aall.^2, 2)));
    iter = iter + 1;
    fprintf('.');
end
fprintf('finished!\n');

%% store the selected exemplars
%{
N = size(X,2);
Exemplar = cell(1,2);
Aall = cell2mat(A);
a = sum(abs(Aall),2);
ind = find(a~=0);
Y = X(:,find(a~=0));

Exemplar{1} = X(:, ind( find(ind<=N/2) ) );
Exemplar{2} = X(:, ind( find(ind>N/2) ) );
%}

%% show the data and exemplars
% projected space
colorList = { 'c', 'b', 'r', 'g', 'm', 'k', 'y'};
shapeList = {'*', 'o', 'x', '+', 's', 'd', 'v', 'p', '^', '<', '>', 'h'};
figure;
tmp = P'*kernelFunction(X, data{1}, type, tau);

ll = 1:C;

thetaList = [pi/180*30, pi/180*10, pi/180*50, pi/180*80, 0];

for i = 1:length(ll)-3
    c = ll(i);
    colorIdx = mod(c, 7)+1;
    shapeIdx = mod(c, 12)+1;
    tmp = P'*kernelFunction(X, data{c}, type, tau);
    
    theta = thetaList(i);
    rotProj = [cos(theta), -sin(theta); sin(theta), cos(theta)];
    tmp(2:3,:) = rotProj*tmp(2:3,:);
    plot3(tmp(1,:), tmp(2,:),tmp(3,:), [colorList{colorIdx} shapeList{shapeIdx}], 'MarkerSize', 10 ); hold on;
end
grid on;
hold off;
legend('ID-1', 'ID-2', 'ID-3', 'ID-4', 'ID-5', 'ID-6', 'ID-7', 'ID-8', 'ID-9', 'Location', 'EastOutside');
