function Ac = updateAc_alternative(P, X, labelList, c, lambda, opts, type, tau)
% This function is used for updating coefficient matrix Ac
%
% Input
%       P           --  the projection matrix;
%       X           --  the training data set of size p x N;
%       labelList	--  the label list for the training set, 1 x N;
%       c           --  the class label indicating which coefficient matrix
%                       need to be updated;
%       lambda      --  the parameter used for tune the L1,2 sparsity;
%       opts        --  Optional inputs (default value: opts=[]), see
%                       function 'mcLeastR'
%
% Output
%       Ac       -- the updated coefficient matrix Ac.
%
% Author: Shu (Aimery) Kong
% Date:   Jan. 19, 2013

K = kernelFunction(X, X, type, tau);
ind = find(labelList==c);
Q = createSelectionMatrix(ind, length(labelList));
Q = Q*Q';

Xc = X(:,ind);
Xc = kernelFunction(X, Xc, type, tau);
Xc = P'*Xc;
Xc = [Xc; Xc; zeros(size(Xc))];

ind = find(labelList~=c);
Qbar = createSelectionMatrix(ind, length(labelList));
Qbar = Qbar*Qbar';
Xbar = P'*K;
Xbar = [Xbar; Xbar*Q; Xbar*Qbar];

[Ac, funVal, ValueL]= mcLeastR(Xbar, Xc, lambda, opts);