function [P, Aall, Ahat, Abar] = updateP(P, X, A, labelList, dimP, type, tau)
% This function is used for updating projection W
%
% Input
%       P           --  the alternative projection matrix, such that X*P is the real one;
%       X           --  the training data set of size p x N;
%       labelList	--  the label list for the training set, 1 x N;
%       dimW        --  the class label indicating which coefficient matrix
%
% Output
%       P           -- the updated projection P.
%
% Author: Shu (Aimery) Kong
% Date:   Jan. 19, 2013

K = kernelFunction(X, X, type, tau);
Aall = cell2mat(A);
Ahat = [];
Abar = [];
C = length(unique(labelList));

for c = 1:C    
    ind = find(labelList==c);
    Q = createSelectionMatrix(ind, length(labelList));
    Q = Q*Q';
    Ahat = [Ahat, Q*A{c}];
    
    ind = find(labelList~=c);
    Q = createSelectionMatrix(ind, length(labelList));
    Q = Q*Q';
    Abar = [Abar, Q*A{c}];
end

%%
tmp = eye(size(X,2)) - Aall;
H = tmp*tmp';
tmp = eye(size(X,2)) - Ahat;
H = H + tmp*tmp';
H = H + Abar*Abar';

[U, S, V] = svd(K);
tmp = diag(S);
tmp = sqrt(tmp);
tmp = diag(tmp);
H = tmp*U'*H*U*tmp;

tmp = diag(S);
tmp = -1.*sqrt(tmp);
tmp = diag(tmp);

[G, S] = eig(H);
G = G(:, 1:dimP);
P = U*tmp*G;
